//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#define MAGICMA  20050610

extern double Lots               = 0.1;
extern double MaximumRisk        = 0.02;
extern double DecreaseFactor     = 3;
extern double MovingPeriod       = 12;
extern double MovingShift        = 6;

int OrderCount[6];

//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
int CheckForOpen() {
   double ma;
   int    res=-1;
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
   if(Open[1]>ma && Close[1]<ma)  res=1;
   if(Open[1]<ma && Close[1]>ma)  res=0;
   return(res);
  }
void ForOpen(string _Symbol, int _Magic,int OP) {
   int    res;
   double price=NormalizeDouble(iif(OP==OP_SELL,Bid,Ask),Digits);
   res=OrderSend(_Symbol,OP,LotsOptimized(),price,3,0,0,"",_Magic,0,iif(OP==OP_SELL,Red,Blue));
  }


//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void ForClose(string _Symbol="", int _Magic=0, int OP=-1) {
   for(int i=0;i<OrdersTotal();i++) {
      if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))   continue;
      if(!(OrderMagicNumber()==_Magic || _Magic==0))  continue;
      if (!(OrderSymbol()==_Symbol || _Symbol==""))   continue;
      if (!(OrderType()==OP || OP==-1))               continue;
      double price = iif(OrderType()==OP_BUY,Bid, Ask);
      if(OrderClose(OrderTicket(),OrderLots(),price,3,White)) break;
   }
}


//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start() {
   if(Bars<100 || IsTradeAllowed()==false) return;
   if (!CheckHour()) return(0);
   int cmd=CheckForOpen();
   if (cmd==-1) return(0);
   int rcmd=MathAbs(cmd-1);
   CalculateOrderCount(Symbol(), MAGICMA);
   if (OrderCount[rcmd]>0) ForClose(Symbol(), MAGICMA,rcmd);
   CalculateOrderCount(Symbol(), MAGICMA);
   if (OrderCount[rcmd]==0) ForOpen(Symbol(), MAGICMA,cmd);
}




//===============================================================================
//    
//===============================================================================
bool CheckHour(){
   static int PrevTime=0;
   int NevTime=iTime(NULL,PERIOD_H1,0);
   if (NevTime==PrevTime) return(false);
   PrevTime=NevTime;
   return(true);
}

//===============================================================================
//          
//===============================================================================
void CalculateOrderCount(string _Symbol="", int _Magic=0, int _OP=-1){
   ArrayInitialize(OrderCount,0);
   for (int i = OrdersTotal() - 1;  i >= 0;  i--) {
      if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES))   continue;
      if (!(OrderSymbol() == _Symbol || _Symbol==""))    continue;
      if (!(OrderMagicNumber() == _Magic || _Magic==0))  continue;
      if (!(OrderType()==_OP || _OP==-1))                continue;
      OrderCount[OrderType()]++;
   }
}


//======================================================================================================
//   
//=====================================================================================================
double iif(bool A, double B, double C) {if(A) return(B); return(C); }